@@ -80,6 +80,7 @@ func (a byTimeOrder) Less(i, j int) bool {
 
 var _spinner = []string{`-`, `\`, `|`, `/`, `-`, `\`, `|`, `/`}
 var _runeWidths = make(map[rune]int)
+var _tabStop int
 
 const (
 	reqPrompt util.EventType = iota
@@ -132,6 +133,7 @@ const (
 	actPreviousHistory
 	actNextHistory
 	actExecute
+	actExecuteMulti
 )
 
 func defaultKeymap() map[int]actionType {
@@ -193,6 +195,7 @@ func NewTerminal(opts *Options, eventBox *util.EventBox) *Terminal {
 	} else {
 		header = reverseStringArray(opts.Header)
 	}
+	_tabStop = opts.Tabstop
 	return &Terminal{
 		inlineInfo: opts.InlineInfo,
 		prompt:     opts.Prompt,
@@ -305,21 +308,25 @@ func (t *Terminal) output() bool {
 			found = true
 		}
 	} else {
-		sels := make([]selectedItem, 0, len(t.selected))
-		for _, sel := range t.selected {
-			sels = append(sels, sel)
-		}
-		sort.Sort(byTimeOrder(sels))
-		for _, sel := range sels {
+		for _, sel := range t.sortSelected() {
 			fmt.Println(*sel.text)
 		}
 	}
 	return found
 }
 
+func (t *Terminal) sortSelected() []selectedItem {
+	sels := make([]selectedItem, 0, len(t.selected))
+	for _, sel := range t.selected {
+		sels = append(sels, sel)
+	}
+	sort.Sort(byTimeOrder(sels))
+	return sels
+}
+
 func runeWidth(r rune, prefixWidth int) int {
 	if r == '\t' {
-		return 8 - prefixWidth%8
+		return _tabStop - prefixWidth%_tabStop
 	} else if w, found := _runeWidths[r]; found {
 		return w
 	} else {
@@ -391,7 +398,7 @@ func (t *Terminal) move(y int, x int, clear bool) {
 }
 
 func (t *Terminal) placeCursor() {
-	t.move(0, len(t.prompt)+displayWidth(t.input[:t.cx]), false)
+	t.move(0, displayWidth([]rune(t.prompt))+displayWidth(t.input[:t.cx]), false)
 }
 
 func (t *Terminal) printPrompt() {
@@ -402,7 +409,7 @@ func (t *Terminal) printPrompt() {
 
 func (t *Terminal) printInfo() {
 	if t.inlineInfo {
-		t.move(0, len(t.prompt)+displayWidth(t.input)+1, true)
+		t.move(0, displayWidth([]rune(t.prompt))+displayWidth(t.input)+1, true)
 		if t.reading {
 			C.CPrint(C.ColSpinner, true, " < ")
 		} else {
@@ -698,8 +705,12 @@ func keyMatch(key int, event C.Event) bool {
 	return event.Type == key || event.Type == C.Rune && int(event.Char) == key-C.AltZ
 }
 
-func executeCommand(template string, current string) {
-	command := strings.Replace(template, "{}", fmt.Sprintf("%q", current), -1)
+func quoteEntry(entry string) string {
+	return fmt.Sprintf("%q", entry)
+}
+
+func executeCommand(template string, replacement string) {
+	command := strings.Replace(template, "{}", replacement, -1)
 	cmd := exec.Command("sh", "-c", command)
 	cmd.Stdin = os.Stdin
 	cmd.Stdout = os.Stdout
@@ -858,7 +869,17 @@ func (t *Terminal) Loop() {
 			case actExecute:
 				if t.cy >= 0 && t.cy < t.merger.Length() {
 					item := t.merger.Get(t.cy)
-					executeCommand(t.execmap[mapkey], item.AsString(t.ansi))
+					executeCommand(t.execmap[mapkey], quoteEntry(item.AsString(t.ansi)))
+				}
+			case actExecuteMulti:
+				if len(t.selected) > 0 {
+					sels := make([]string, len(t.selected))
+					for i, sel := range t.sortSelected() {
+						sels[i] = quoteEntry(*sel.text)
+					}
+					executeCommand(t.execmap[mapkey], strings.Join(sels, " "))
+				} else {
+					return doAction(actExecute, mapkey)
 				}
 			case actInvalid:
 				t.mutex.Unlock()
@@ -1023,7 +1044,7 @@ func (t *Terminal) Loop() {
 					my >= t.marginInt[0] && my < C.MaxY()-t.marginInt[2] {
 					mx -= t.marginInt[3]
 					my -= t.marginInt[0]
-					mx = util.Constrain(mx-len(t.prompt), 0, len(t.input))
+					mx = util.Constrain(mx-displayWidth([]rune(t.prompt)), 0, len(t.input))
 					if !t.reverse {
 						my = t.maxHeight() - my - 1
 					}
